@@ -1,11 +1,10 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2009, University of Amsterdam
+    Copyright (C): 1985-2013, University of Amsterdam
+			      Vu University Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -27,6 +26,7 @@
 
 #include <SWI-Prolog.h>
 #include <string.h>
+#include <wchar.h>
 #ifndef __APPLE__
 #include <malloc.h>
 #endif
@@ -56,6 +56,9 @@ public:
   PlFunctor(const char *name, int arity)
   { functor = PL_new_functor(PL_new_atom(name), arity);
   }
+  PlFunctor(const wchar_t *name, int arity)
+  { functor = PL_new_functor(PL_new_atom_wchars(wcslen(name), name), arity);
+  }
 };
 
 
@@ -70,15 +73,24 @@ public:
   PlAtom(const char *text)
   { handle = PL_new_atom(text);
   }
+  PlAtom(const wchar_t *text)
+  { handle = PL_new_atom_wchars(wcslen(text), text);
+  }
   PlAtom(const PlTerm &t);
 
   operator const char *(void)
   { return PL_atom_chars(handle);
   }
+  operator const wchar_t *(void)
+  { return PL_atom_wchars(handle, NULL);
+  }
 
   int operator ==(const char *s)
   { return strcmp(s, PL_atom_chars(handle)) == 0;
   }
+  int operator ==(const wchar_t *s)
+  { return wcscmp(s, PL_atom_wchars(handle, NULL)) == 0;
+  }
   int operator ==(const PlAtom &a)
   { return handle == a.handle;
   }
@@ -101,6 +113,7 @@ public:
 
 					/* C --> PlTerm */
   PlTerm(const char *text);
+  PlTerm(const wchar_t *text);
   PlTerm(long val);
   PlTerm(double val);
   PlTerm(const PlAtom &a);
@@ -111,6 +124,7 @@ public:
   { return ref;
   }
   operator char *(void) const;
+  operator wchar_t *(void) const;
   operator long(void) const;
   operator int(void) const;
   operator double(void) const;
@@ -130,6 +144,7 @@ public:
   int operator =(const PlTerm &t2);	/* term */
   int operator =(const PlAtom &a);	/* atom */
   int operator =(const char *v);	/* atom (from char *) */
+  int operator =(const wchar_t *v);	/* atom (from wchar_t *) */
   int operator =(long v);		/* integer */
   int operator =(int v);		/* integer */
   int operator =(double v);		/* float */
@@ -164,6 +179,7 @@ public:
 
 					/* comparison (string) */
   int operator ==(const char *s);
+  int operator ==(const wchar_t *s);
   int operator ==(const PlAtom &a);
 };
 
@@ -206,7 +222,9 @@ class PlCompound : public PlTerm
 public:
 
   PlCompound(const char *text);
+  PlCompound(const wchar_t *text);
   PlCompound(const char *functor, const PlTermv &args);
+  PlCompound(const wchar_t *functor, const PlTermv &args);
 };
 
 
@@ -215,7 +233,9 @@ class PlString : public PlTerm
 public:
 
   PlString(const char *text);
-  PlString(const char *text, int len);
+  PlString(const char *text, size_t len);
+  PlString(const wchar_t *text);
+  PlString(const wchar_t *text, size_t len);
 };
 
 
@@ -224,6 +244,7 @@ class PlCodeList : public PlTerm
 public:
 
   PlCodeList(const char *text);
+  PlCodeList(const wchar_t *text);
 };
 
 
@@ -232,6 +253,7 @@ class PlCharList : public PlTerm
 public:
 
   PlCharList(const char *text);
+  PlCharList(const wchar_t *text);
 };
 
 
@@ -255,6 +277,7 @@ public:
   }
 
   operator const char *(void);
+  operator const wchar_t *(void);
 
   int plThrow()
   { return PL_raise_exception(ref);
@@ -346,6 +369,13 @@ PlTerm::PlTerm(const char *text)
     throw PlResourceError();
 }
 
+__inline
+PlTerm::PlTerm(const wchar_t *text)
+{ if ( !(ref = PL_new_term_ref()) ||
+       !PL_unify_wchars(ref, PL_ATOM, (size_t)-1, text) )
+    throw PlResourceError();
+}
+
 __inline
 PlTerm::PlTerm(long val)
 { if ( !(ref = PL_new_term_ref()) ||
@@ -387,11 +417,23 @@ PlString::PlString(const char *text) : PlTerm()
 }
 
 __inline
-PlString::PlString(const char *text, int len) : PlTerm()
+PlString::PlString(const char *text, size_t len) : PlTerm()
 { if ( !PL_put_string_nchars(ref, len, text) )
     throw PlResourceError();
 }
 
+__inline
+PlString::PlString(const wchar_t *text) : PlTerm()
+{ if ( !PL_unify_wchars(ref, PL_STRING, (size_t)-1, text) )
+    throw PlResourceError();
+}
+
+__inline
+PlString::PlString(const wchar_t *text, size_t len) : PlTerm()
+{ if ( !PL_unify_wchars(ref, PL_STRING, len, text) )
+    throw PlResourceError();
+}
+
 __inline
 PlCodeList::PlCodeList(const char *text) : PlTerm()
 { if ( !PL_put_list_codes(ref, text) )
@@ -404,6 +446,18 @@ PlCharList::PlCharList(const char *text) : PlTerm()
     throw PlResourceError();
 }
 
+__inline
+PlCodeList::PlCodeList(const wchar_t *text) : PlTerm()
+{ if ( !PL_unify_wchars(ref, PL_CODE_LIST, (size_t)-1, text) )
+    throw PlResourceError();
+}
+
+__inline
+PlCharList::PlCharList(const wchar_t *text) : PlTerm()
+{ if ( !PL_unify_wchars(ref, PL_CHAR_LIST, (size_t)-1, text) )
+    throw PlResourceError();
+}
+
 
 		 /*******************************
 		 *             LISTS		*
@@ -563,6 +617,12 @@ PlCall(const char *goal)
   return q.next_solution();
 }
 
+__inline int
+PlCall(const wchar_t *goal)
+{ PlQuery q("call", PlTermv(PlCompound(goal)));
+  return q.next_solution();
+}
+
 
 
 		 /*******************************
@@ -589,7 +649,17 @@ PlAtom::PlAtom(const PlTerm &t)
 __inline PlTerm::operator char *(void) const
 { char *s;
 
-  if ( PL_get_chars(ref, &s, CVT_ALL|CVT_WRITE|BUF_RING) )
+  if ( PL_get_chars(ref, &s, CVT_ALL|CVT_WRITEQ|BUF_RING) )
+    return s;
+
+  throw PlTypeError("text", ref);
+  PL_THROWN(NULL);
+}
+
+__inline PlTerm::operator wchar_t *(void) const
+{ wchar_t *s;
+
+  if ( PL_get_wchars(ref, NULL, &s, CVT_ALL|CVT_WRITEQ|BUF_RING) )
     return s;
 
   throw PlTypeError("text", ref);
@@ -707,7 +777,7 @@ __inline int PlTerm::operator =(const PlTerm &t2)	/* term = term */
   return rc;
 }
 
-__inline int PlTerm::operator =(const PlAtom &a) 	/* term = atom */
+__inline int PlTerm::operator =(const PlAtom &a)	/* term = atom */
 { int rc = PL_unify_atom(ref, a.handle);
   term_t ex;
 
@@ -725,6 +795,15 @@ __inline int PlTerm::operator =(const char *v)		/* term = atom */
   return rc;
 }
 
+__inline int PlTerm::operator =(const wchar_t *v)	/* term = atom */
+{ int rc = PL_unify_wchars(ref, PL_ATOM, (size_t)-1, v);
+  term_t ex;
+
+  if ( !rc && (ex=PL_exception(0)) )
+    throw PlResourceError(ex);
+  return rc;
+}
+
 __inline int PlTerm::operator =(long v)
 { int rc = PL_unify_integer(ref, v);
   term_t ex;
@@ -836,6 +915,15 @@ __inline int PlTerm::operator ==(const char *s)
   PL_THROWN(0);
 }
 
+__inline int PlTerm::operator ==(const wchar_t *s)
+{ wchar_t *s0;
+
+  if ( PL_get_wchars(ref, NULL, &s0, CVT_ALL) )
+    return wcscmp(s0, s) == 0;
+
+  throw PlTypeError("text", ref);
+  PL_THROWN(0);
+}
 
 __inline int PlTerm::operator ==(const PlAtom &a)
 { atom_t v;
@@ -849,7 +937,7 @@ __inline int PlTerm::operator ==(const PlAtom &a)
 
 
 		 /*******************************
-		 *	COMPPOUND (BODY)	*
+		 *	   COMPOUND (BODY)	*
 		 *******************************/
 
 
@@ -863,6 +951,16 @@ PlCompound::PlCompound(const char *text) : PlTerm()
   PL_put_term(ref, t);
 }
 
+__inline
+PlCompound::PlCompound(const wchar_t *text) : PlTerm()
+{ term_t t = PL_new_term_ref();
+
+  if ( !PL_wchars_to_term(text, t) )
+    throw PlException(t);
+
+  PL_put_term(ref, t);
+}
+
 __inline
 PlCompound::PlCompound(const char *functor, const PlTermv &args) : PlTerm()
 { if ( !PL_cons_functor_v(ref,
@@ -871,6 +969,16 @@ PlCompound::PlCompound(const char *functor, const PlTermv &args) : PlTerm()
     throw PlResourceError();
 }
 
+__inline
+PlCompound::PlCompound(const wchar_t *functor, const PlTermv &args) : PlTerm()
+{ if ( !PL_cons_functor_v(
+	    ref,
+	    PL_new_functor(PL_new_atom_wchars(wcslen(functor), functor),
+			   args.size),
+	    args.a0) )
+    throw PlResourceError();
+}
+
 		 /*******************************
 		 *	   TERMV (BODY)		*
 		 *******************************/
@@ -956,6 +1064,27 @@ __inline PlException::operator const char *(void)
 }
 
 
+__inline PlException::operator const wchar_t *(void)
+{ PlFrame fr;
+#ifdef USE_PRINT_MESSAGE
+  PlTermv av(2);
+
+  av[0] = PlCompound("print_message",
+		     PlTermv("error", ref));
+  PlQuery q("$write_on_string", av);
+  if ( q.next_solution() )
+    return (wchar_t *)av[1];
+#else
+  PlTermv av(2);
+  av[0] = PlTerm(ref);
+  PlQuery q("$messages", "message_to_string", av);
+  if ( q.next_solution() )
+    return (wchar_t *)av[1];
+#endif
+  return L"[ERROR: Failed to generate message.  Internal error]\n";
+}
+
+
 __inline void
 PlException::cppThrow()
 { term_t a = PL_new_term_ref();
@@ -1006,8 +1135,16 @@ public:
   char *message;
 
   PlError(const char *msg)
-  { message = new char[strlen(msg+1)];
-    strcpy(message, msg);
+  { size_t len = strlen(msg)+1;
+    message = new char[len];
+#ifdef _MSC_VER				/* Yek */
+#pragma warning( push )
+#pragma warning (disable:4996)
+#endif
+    strncpy(message, msg, len);
+#ifdef _MSC_VER
+#pragma warning( pop )
+#endif
   }
 };
 
@@ -1045,49 +1182,84 @@ public:
 #define PROLOG_MODULE (const char*)NULL
 #endif
 
-#define PREDICATE(name, arity) \
+#define NAMED_PREDICATE(plname, name, arity) \
 	static foreign_t \
-	pl_ ## name ## __ ## arity(PlTermv _av); \
+	pl_ ## name ## __ ## arity(PlTermv PL_av); \
 	static foreign_t \
 	_pl_ ## name ## __ ## arity(term_t t0, int a, control_t c) \
-	{ try \
+	{ (void)a; (void)c; \
+          try \
 	  { \
 	    return pl_ ## name ## __ ## arity(PlTermv(arity, t0)); \
 	  } catch ( PlException &ex ) \
 	  { return ex.plThrow(); \
 	  } \
 	} \
-	static PlRegister _x ## name ## __ ## arity(PROLOG_MODULE, #name, arity, \
+	static PlRegister _x ## name ## __ ## arity(PROLOG_MODULE, plname, arity, \
 					    _pl_ ## name ## __ ## arity); \
-	static foreign_t pl_ ## name ## __ ## arity(PlTermv _av)
+	static foreign_t pl_ ## name ## __ ## arity(PlTermv PL_av)
 
-
-#define PREDICATE_NONDET(name, arity)          \
+#define NAMED_PREDICATE0(plname, name) \
 	static foreign_t \
-	pl_ ## name ## __ ## arity(PlTermv _av, foreign_t handle);       \
+	pl_ ## name ## __0(void); \
+	static foreign_t \
+	_pl_ ## name ## __0(term_t t0, int a, control_t c) \
+	{ (void)t0; (void)a; (void)c; \
+          try \
+	  { \
+	    return pl_ ## name ## __0(); \
+	  } catch ( PlException &ex ) \
+	  { return ex.plThrow(); \
+	  } \
+	} \
+	static PlRegister _x ## name ## __0(PROLOG_MODULE, plname, 0, \
+					    _pl_ ## name ## __0); \
+	static foreign_t pl_ ## name ## __0(void)
+
+#define NAMED_PREDICATE_NONDET(plname, name, arity)          \
+	static foreign_t \
+	pl_ ## name ## __ ## arity(PlTermv PL_av, foreign_t handle);       \
 	static foreign_t \
 	_pl_ ## name ## __ ## arity(term_t t0, int a, control_t c) \
-	{ try \
+	{ (void)a; \
+          try \
 	  { \
 	    return pl_ ## name ## __ ## arity(PlTermv(arity, t0), (foreign_t)c); \
 	  } catch ( PlException &ex ) \
 	  { return ex.plThrow(); \
 	  } \
 	} \
-        static PlRegister _x ## name ## __ ## arity(PROLOG_MODULE, #name, arity, \
+        static PlRegister _x ## name ## __ ## arity(PROLOG_MODULE, plname, arity, \
                                                     _pl_ ## name ## __ ## arity, \
                                                     PL_FA_NONDETERMINISTIC | PL_FA_VARARGS); \
-	static foreign_t pl_ ## name ## __ ## arity(PlTermv _av, foreign_t handle)
+	static foreign_t pl_ ## name ## __ ## arity(PlTermv PL_av, foreign_t handle)
 
-#define A1  _av[0]
-#define A2  _av[1]
-#define A3  _av[2]
-#define A4  _av[3]
-#define A5  _av[4]
-#define A6  _av[5]
-#define A7  _av[6]
-#define A8  _av[7]
-#define A9  _av[8]
-#define A10 _av[9]
+#define PREDICATE0(name)              NAMED_PREDICATE0(#name, name)
+#define PREDICATE(name, arity)        NAMED_PREDICATE(#name, name, arity)
+#define PREDICATE_NONDET(name, arity) NAMED_PREDICATE_NONDET(#name, name, arity)
+
+#define PL_A1  PL_av[0]
+#define PL_A2  PL_av[1]
+#define PL_A3  PL_av[2]
+#define PL_A4  PL_av[3]
+#define PL_A5  PL_av[4]
+#define PL_A6  PL_av[5]
+#define PL_A7  PL_av[6]
+#define PL_A8  PL_av[7]
+#define PL_A9  PL_av[8]
+#define PL_A10 PL_av[9]
+
+#ifndef PL_SAFE_ARG_MACROS
+#define A1	PL_A1
+#define A2	PL_A2
+#define A3	PL_A3
+#define A4	PL_A4
+#define A5	PL_A5
+#define A6	PL_A6
+#define A7	PL_A7
+#define A8	PL_A8
+#define A9	PL_A9
+#define A10	PL_A10
+#endif
 
 #endif /*_SWI_CPP_H*/
