@@ -67,6 +67,15 @@ index((scanl), 4, apply, apply).
 index((scanl), 5, apply, apply).
 index((scanl), 6, apply, apply).
 index((scanl), 7, apply, apply).
+index((expand_phrase), 2, apply_macros, apply_macros).
+index((archive_open), 3, archive, archive).
+index((archive_close), 1, archive, archive).
+index((archive_property), 2, archive, archive).
+index((archive_next_header), 2, archive, archive).
+index((archive_open_entry), 2, archive, archive).
+index((archive_header_property), 2, archive, archive).
+index((archive_extract), 3, archive, archive).
+index((archive_entries), 2, archive, archive).
 index((arithmetic_function), 1, arithmetic, arithmetic).
 index((arithmetic_expression_value), 2, arithmetic, arithmetic).
 index((empty_assoc), 1, assoc, assoc).
@@ -102,6 +111,7 @@ index((sformat), 3, backward_compatibility, backcomp).
 index((concat), 3, backward_compatibility, backcomp).
 index((concat_atom), 2, backward_compatibility, backcomp).
 index((concat_atom), 3, backward_compatibility, backcomp).
+index(('$apropos_match'), 2, backward_compatibility, backcomp).
 index((read_clause), 1, backward_compatibility, backcomp).
 index((read_clause), 2, backward_compatibility, backcomp).
 index((read_variables), 2, backward_compatibility, backcomp).
@@ -109,6 +119,8 @@ index((read_variables), 3, backward_compatibility, backcomp).
 index((feature), 2, backward_compatibility, backcomp).
 index((set_feature), 2, backward_compatibility, backcomp).
 index((substring), 4, backward_compatibility, backcomp).
+index((string_to_list), 2, backward_compatibility, backcomp).
+index((string_to_atom), 2, backward_compatibility, backcomp).
 index((flush), 0, backward_compatibility, backcomp).
 index((write_ln), 1, backward_compatibility, backcomp).
 index((proper_list), 1, backward_compatibility, backcomp).
@@ -168,6 +180,8 @@ index((list_undefined), 0, check, check).
 index((list_undefined), 1, check, check).
 index((list_autoload), 0, check, check).
 index((list_redefined), 0, check, check).
+index((list_trivial_fails), 0, check, check).
+index((list_trivial_fails), 1, check, check).
 index((check_old_last), 0, checklast, checklast).
 index((check_old_select), 0, checkselect, checkselect).
 index((chr_show_store), 1, chr, chr).
@@ -293,9 +307,13 @@ index((gensym), 2, gensym, gensym).
 index((git), 2, git, git).
 index((git_process_output), 3, git, git).
 index((git_open_file), 4, git, git).
+index((is_git_directory), 1, git, git).
 index((git_describe), 2, git, git).
+index((git_hash), 2, git, git).
+index((git_ls_tree), 2, git, git).
 index((git_remote_url), 3, git, git).
 index((git_ls_remote), 3, git, git).
+index((git_branches), 2, git, git).
 index((git_remote_branches), 2, git, git).
 index((git_default_branch), 2, git, git).
 index((git_tags_on_branch), 3, git, git).
@@ -397,6 +415,34 @@ index((occurrences_of_term), 3, occurs, occurs).
 index((occurrences_of_var), 3, occurs, occurs).
 index((sub_term), 2, occurs, occurs).
 index((sub_var), 2, occurs, occurs).
+index((odbc_connect), 3, odbc, odbc).
+index((odbc_driver_connect), 3, odbc, odbc).
+index((odbc_disconnect), 1, odbc, odbc).
+index((odbc_current_connection), 2, odbc, odbc).
+index((odbc_set_connection), 2, odbc, odbc).
+index((odbc_get_connection), 2, odbc, odbc).
+index((odbc_end_transaction), 2, odbc, odbc).
+index((odbc_query), 4, odbc, odbc).
+index((odbc_query), 3, odbc, odbc).
+index((odbc_query), 2, odbc, odbc).
+index((odbc_prepare), 4, odbc, odbc).
+index((odbc_prepare), 5, odbc, odbc).
+index((odbc_execute), 2, odbc, odbc).
+index((odbc_execute), 3, odbc, odbc).
+index((odbc_fetch), 3, odbc, odbc).
+index((odbc_close_statement), 1, odbc, odbc).
+index((odbc_clone_statement), 2, odbc, odbc).
+index((odbc_free_statement), 1, odbc, odbc).
+index((odbc_current_table), 2, odbc, odbc).
+index((odbc_current_table), 3, odbc, odbc).
+index((odbc_table_column), 3, odbc, odbc).
+index((odbc_table_column), 4, odbc, odbc).
+index((odbc_type), 3, odbc, odbc).
+index((odbc_data_source), 2, odbc, odbc).
+index((odbc_table_primary_key), 3, odbc, odbc).
+index((odbc_table_foreign_key), 5, odbc, odbc).
+index((odbc_statistics), 1, odbc, odbc).
+index((odbc_debug), 1, odbc, odbc).
 index((push_operators), 1, prolog_operator, operators).
 index((push_operators), 2, prolog_operator, operators).
 index((pop_operators), 0, prolog_operator, operators).
@@ -500,6 +546,7 @@ index((initialization_layout), 4, prolog_clause, prolog_clause).
 index((predicate_name), 2, prolog_clause, prolog_clause).
 index((clause_name), 2, prolog_clause, prolog_clause).
 index((prolog_walk_code), 1, prolog_codewalk, prolog_codewalk).
+index((prolog_program_clause), 2, prolog_codewalk, prolog_codewalk).
 index((prolog_colourise_stream), 3, prolog_colour, prolog_colour).
 index((prolog_colourise_term), 4, prolog_colour, prolog_colour).
 index((syntax_colour), 2, prolog_colour, prolog_colour).
@@ -524,6 +571,7 @@ index((read_source_term_at_location), 3, prolog_source, prolog_source).
 index((prolog_open_source), 2, prolog_source, prolog_source).
 index((prolog_close_source), 1, prolog_source, prolog_source).
 index((prolog_canonical_source), 2, prolog_source, prolog_source).
+index((load_quasi_quotation_syntax), 2, prolog_source, prolog_source).
 index((file_name_on_path), 2, prolog_source, prolog_source).
 index((file_alias_path), 2, prolog_source, prolog_source).
 index((path_segments_atom), 2, prolog_source, prolog_source).
@@ -535,6 +583,7 @@ index((print_prolog_backtrace), 2, prolog_stack, prolog_stack).
 index((print_prolog_backtrace), 3, prolog_stack, prolog_stack).
 index((backtrace), 1, prolog_stack, prolog_stack).
 index((xref_source), 1, prolog_xref, prolog_xref).
+index((xref_source), 2, prolog_xref, prolog_xref).
 index((xref_called), 3, prolog_xref, prolog_xref).
 index((xref_called), 4, prolog_xref, prolog_xref).
 index((xref_defined), 3, prolog_xref, prolog_xref).
@@ -543,15 +592,21 @@ index((xref_exported), 2, prolog_xref, prolog_xref).
 index((xref_module), 2, prolog_xref, prolog_xref).
 index((xref_uses_file), 3, prolog_xref, prolog_xref).
 index((xref_op), 2, prolog_xref, prolog_xref).
+index((xref_comment), 3, prolog_xref, prolog_xref).
+index((xref_comment), 4, prolog_xref, prolog_xref).
+index((xref_mode), 3, prolog_xref, prolog_xref).
+index((xref_option), 2, prolog_xref, prolog_xref).
 index((xref_clean), 1, prolog_xref, prolog_xref).
 index((xref_current_source), 1, prolog_xref, prolog_xref).
 index((xref_done), 2, prolog_xref, prolog_xref).
 index((xref_built_in), 1, prolog_xref, prolog_xref).
 index((xref_source_file), 3, prolog_xref, prolog_xref).
 index((xref_source_file), 4, prolog_xref, prolog_xref).
+index((xref_public_list), 3, prolog_xref, prolog_xref).
 index((xref_public_list), 4, prolog_xref, prolog_xref).
 index((xref_public_list), 6, prolog_xref, prolog_xref).
 index((xref_public_list), 7, prolog_xref, prolog_xref).
+index((xref_meta), 3, prolog_xref, prolog_xref).
 index((xref_meta), 2, prolog_xref, prolog_xref).
 index((xref_hook), 1, prolog_xref, prolog_xref).
 index((xref_used_class), 2, prolog_xref, prolog_xref).
@@ -560,6 +615,10 @@ index((protobuf_message), 2, protobufs, protobufs).
 index((protobuf_message), 3, protobufs, protobufs).
 index((phrase_from_file), 2, pure_input, pure_input).
 index((phrase_from_file), 3, pure_input, pure_input).
+index((syntax_error), 3, pure_input, pure_input).
+index((lazy_list_location), 3, pure_input, pure_input).
+index((lazy_list_character_count), 3, pure_input, pure_input).
+index((phrase_from_stream), 2, pure_input, pure_input).
 index((stream_to_lazy_list), 2, pure_input, pure_input).
 index((pwp_files), 2, pwp, pwp).
 index((pwp_stream), 3, pwp, pwp).
@@ -574,6 +633,10 @@ index((make_foreign_resource_wrapper), 3, qp_foreign, qpforeign).
 index((load_foreign_resource), 2, qp_foreign, qpforeign).
 index((qsave_program), 1, qsave, qsave).
 index((qsave_program), 2, qsave, qsave).
+index((with_quasi_quotation_input), 3, quasi_quotations, quasi_quotations).
+index((phrase_from_quasi_quotation), 2, quasi_quotations, quasi_quotations).
+index((quasi_quotation_syntax_error), 1, quasi_quotations, quasi_quotations).
+index((quasi_quotation_syntax), 1, quasi_quotations, quasi_quotations).
 index((unix), 1, quintus, quintus).
 index((abs), 2, quintus, quintus).
 index((sin), 2, quintus, quintus).
@@ -659,8 +722,8 @@ index((load_rdf), 2, rdf, rdf).
 index((load_rdf), 3, rdf, rdf).
 index((xml_to_rdf), 3, rdf, rdf).
 index((process_rdf), 3, rdf, rdf).
-index((load_rdf_ntriples), 2, rdf_ntriples, rdf_ntriples).
-index((rdf_ntriple_part), 4, rdf_ntriples, rdf_ntriples).
+index((load_rdf_ntriples), 2, rdf_ntriples_old, rdf_ntriples).
+index((rdf_ntriple_part), 4, rdf_ntriples_old, rdf_ntriples).
 index((xml_to_plrdf), 3, rdf_parser, rdf_parser).
 index((element_to_plrdf), 3, rdf_parser, rdf_parser).
 index((make_rdf_state), 3, rdf_parser, rdf_parser).
@@ -689,6 +752,7 @@ index((rewrite), 2, rewrite, rewrite).
 index((rew_term_expansion), 2, rewrite, rewrite).
 index((rew_goal_expansion), 2, rewrite, rewrite).
 index((rlimit), 3, clib_rlimit, rlimit).
+index((safe_goal), 1, sandbox, sandbox).
 index((setting), 4, settings, settings).
 index((setting), 2, settings, settings).
 index((set_setting), 2, settings, settings).
@@ -703,6 +767,9 @@ index((setting_property), 2, settings, settings).
 index((list_settings), 0, settings, settings).
 index((list_settings), 1, settings, settings).
 index((convert_setting_text), 3, settings, settings).
+index((load_html), 3, sgml, sgml).
+index((load_xml), 3, sgml, sgml).
+index((load_sgml), 3, sgml, sgml).
 index((load_sgml_file), 2, sgml, sgml).
 index((load_xml_file), 2, sgml, sgml).
 index((load_html_file), 2, sgml, sgml).
@@ -744,6 +811,7 @@ index((sha_hash), 3, crypto_hash, sha).
 index((sha_new_ctx), 2, crypto_hash, sha).
 index((sha_hash_ctx), 4, crypto_hash, sha).
 index((hmac_sha), 4, crypto_hash, sha).
+index((file_sha1), 2, crypto_hash, sha).
 index((hash_atom), 2, crypto_hash, sha).
 index((ls), 0, shell, shell).
 index((ls), 1, shell, shell).
@@ -770,6 +838,7 @@ index((tcp_socket), 1, socket, socket).
 index((tcp_close_socket), 1, socket, socket).
 index((tcp_open_socket), 3, socket, socket).
 index((tcp_connect), 2, socket, socket).
+index((tcp_connect), 3, socket, socket).
 index((tcp_connect), 4, socket, socket).
 index((tcp_bind), 2, socket, socket).
 index((tcp_accept), 3, socket, socket).
@@ -779,6 +848,7 @@ index((tcp_setopt), 2, socket, socket).
 index((tcp_host_to_address), 2, socket, socket).
 index((tcp_select), 3, socket, socket).
 index((gethostname), 1, socket, socket).
+index((tcp_open_socket), 2, socket, socket).
 index((udp_socket), 1, socket, socket).
 index((udp_receive), 4, socket, socket).
 index((udp_send), 4, socket, socket).
@@ -803,8 +873,7 @@ index((ssl_session), 2, ssl, ssl).
 index((statistics), 0, prolog_statistics, statistics).
 index((time), 1, prolog_statistics, statistics).
 index((profile), 1, prolog_statistics, statistics).
-index((profile), 3, prolog_statistics, statistics).
-index((show_profile), 2, prolog_statistics, statistics).
+index((profile), 2, prolog_statistics, statistics).
 index((show_profile), 1, prolog_statistics, statistics).
 index((stream_info), 1, stream_info, streaminfo).
 index((add_stream_to_pool), 2, stream_pool, streampool).
@@ -812,6 +881,10 @@ index((delete_stream_from_pool), 1, stream_pool, streampool).
 index((close_stream_pool), 0, stream_pool, streampool).
 index((dispatch_stream_pool), 1, stream_pool, streampool).
 index((stream_pool_main_loop), 0, stream_pool, streampool).
+index((openlog), 3, syslog, syslog).
+index((syslog), 2, syslog, syslog).
+index((syslog), 3, syslog, syslog).
+index((closelog), 0, syslog, syslog).
 index((lock_predicate), 1, swi_system_utilities, system).
 index((unlock_predicate), 1, swi_system_utilities, system).
 index((system_mode), 1, swi_system_utilities, system).
@@ -852,6 +925,9 @@ index((show_coverage), 1, prolog_cover, test_cover).
 index((make_tests), 3, test_wizard, test_wizard).
 index((make_test), 3, test_wizard, test_wizard).
 index((concurrent), 3, thread, thread).
+index((concurrent_maplist), 2, thread, thread).
+index((concurrent_maplist), 3, thread, thread).
+index((concurrent_maplist), 4, thread, thread).
 index((first_solution), 3, thread, thread).
 index((thread_pool_create), 3, thread_pool, thread_pool).
 index((thread_pool_destroy), 1, thread_pool, thread_pool).
@@ -862,6 +938,7 @@ index((thread_run_interactor), 0, thread_util, threadutil).
 index((threads), 0, thread_util, threadutil).
 index((join_threads), 0, thread_util, threadutil).
 index((interactor), 0, thread_util, threadutil).
+index((thread_has_console), 0, thread_util, threadutil).
 index((attach_console), 0, thread_util, threadutil).
 index((tspy), 1, thread_util, threadutil).
 index((tspy), 2, thread_util, threadutil).
@@ -925,18 +1002,22 @@ index((unicode_nfkd), 2, unicode, unicode).
 index((unicode_nfkc), 2, unicode, unicode).
 index((fork), 1, unix, unix).
 index((exec), 1, unix, unix).
+index((fork_exec), 1, unix, unix).
 index((wait), 2, unix, unix).
 index((kill), 2, unix, unix).
 index((pipe), 2, unix, unix).
 index((dup), 2, unix, unix).
 index((detach_IO), 0, unix, unix).
+index((detach_IO), 1, unix, unix).
 index((environ), 1, unix, unix).
 index((uri_components), 2, uri, uri).
 index((uri_data), 3, uri, uri).
 index((uri_data), 4, uri, uri).
 index((uri_normalized), 2, uri, uri).
+index((iri_normalized), 2, uri, uri).
 index((uri_normalized_iri), 2, uri, uri).
 index((uri_normalized), 3, uri, uri).
+index((iri_normalized), 3, uri, uri).
 index((uri_normalized_iri), 3, uri, uri).
 index((uri_resolve), 3, uri, uri).
 index((uri_is_global), 1, uri, uri).
@@ -963,6 +1044,7 @@ index((max_var_number), 3, varnumbers, varnumbers).
 index((varnumbers), 3, varnumbers, varnumbers).
 index((vm_list), 1, prolog_vm, vm).
 index((when), 2, when, when).
+index((init_win_menus), 0, win_menu, win_menu).
 index((writef), 1, writef, writef).
 index((writef), 2, writef, writef).
 index((swritef), 2, writef, writef).
