@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -58,9 +58,12 @@ an editor.
 %	Edit indicated object.
 
 edit(Spec) :-
+	notrace(edit_no_trace(Spec)).
+
+edit_no_trace(Spec) :-
 	var(Spec), !,
 	throw(error(instantiation_error, _)).
-edit(Spec) :-
+edit_no_trace(Spec) :-
 	load_extensions,
 	findall(Location-FullSpec,
 		locate(Spec, FullSpec, Location),
@@ -74,18 +77,18 @@ edit(Spec) :-
 %	Edit associated or script file.  This is the Prolog file opened
 %	by double-clicking or the file loaded using
 %
-%	==
-%	% pl -s file.pl
-%	==
+%	  ==
+%	  % swipl [-s] file.pl
+%	  ==
 
+edit :-
+	current_prolog_flag(associated_file, File), !,
+	edit(file(File)).
 edit :-
 	'$option'(script_file, OsFiles),
 	OsFiles = [OsFile], !,
 	prolog_to_os_filename(File, OsFile),
 	edit(file(File)).
-edit :-
-	current_prolog_flag(associated_file, File), !,
-	edit(file(File)).
 edit :-
 	throw(error(context_error(edit, no_default_file), _)).
 
@@ -96,10 +99,27 @@ edit :-
 
 %%	locate(+Spec, -FullSpec, -Location)
 
+locate(FileSpec:Line, file(Path, line(Line)), [file(Path), line(Line)]) :-
+	integer(Line), Line >= 1,
+	ground(FileSpec), !,			% so specific; do not try alts
+	locate(FileSpec, _, [file(Path)]).
+locate(FileSpec:Line:LinePos,
+       file(Path, line(Line), linepos(LinePos)),
+       [file(Path), line(Line), linepos(LinePos)]) :-
+	integer(Line), Line >= 1,
+	integer(LinePos), LinePos >= 1,
+	ground(FileSpec), !,			% so specific; do not try alts
+	locate(FileSpec, _, [file(Path)]).
 locate(Path, file(Path), [file(Path)]) :-
 	atom(Path),
 	exists_file(Path),
 	\+ exists_directory(Path).
+locate(Pattern, file(Path), [file(Path)]) :-
+	atom(Pattern),
+	catch(expand_file_name(Pattern, Files), _, fail),
+	member(Path, Files),
+	exists_file(Path),
+	\+ exists_directory(Path).
 locate(FileBase, file(File), [file(File)]) :-
 	atom(FileBase),
 	absolute_file_name(FileBase,
@@ -121,7 +141,13 @@ locate(FileBase, source_file(Path), [file(Path)]) :-
 	atom(FileBase),
 	source_file(Path),
 	file_base_name(Path, File),
-	file_name_extension(FileBase, pl, File).
+	file_name_extension(FileBase, _, File).
+locate(FileBase, include_file(Path), [file(Path)]) :-
+	atom(FileBase),
+	setof(Path, include_file(Path), Paths),
+	member(Path, Paths),
+	file_base_name(Path, File),
+	file_name_extension(FileBase, _, File).
 locate(Name, FullSpec, Location) :-
 	atom(Name),
 	locate(Name/_, FullSpec, Location).
@@ -162,6 +188,10 @@ locate(Spec, module(Spec), Location) :-
 locate(Spec, Spec, Location) :-
 	locate(Spec, Location).
 
+include_file(Path) :-
+	source_file_property(Path, included_in(_,_)).
+
+
 %%	locate(+Spec, -Location)
 %
 %	Locate object from the specified location.
@@ -169,9 +199,9 @@ locate(Spec, Spec, Location) :-
 locate(file(File, line(Line)), [file(File), line(Line)]).
 locate(file(File), [file(File)]).
 locate(Module:Name/Arity, [file(File), line(Line)]) :-
-	(   nonvar(Arity)
+	(   atom(Name), integer(Arity)
 	->  functor(Head, Name, Arity)
-	;   true
+	;   Head = _			% leave unbound
 	),
 	(   (   var(Module)
 	    ;	var(Name)
@@ -194,6 +224,14 @@ locate(module(Module), [file(Path)|Rest]) :-
 	->  Rest = [line(Line)]
 	;   Rest = []
 	).
+locate(breakpoint(Id), Location) :-
+	integer(Id),
+	breakpoint_property(Id, clause(Ref)),
+	(   breakpoint_property(Id, file(File)),
+	    breakpoint_property(Id, line_count(Line))
+	->  Location = [file(File),line(Line)]
+	;   locate(clause(Ref), Location)
+	).
 locate(clause(Ref), [file(File), line(Line)]) :-
 	clause_property(Ref, file(File)),
 	clause_property(Ref, line_count(Line)).
@@ -226,10 +264,13 @@ do_edit_source(Location) :-		% PceEmacs
 	current_prolog_flag(gui, true), !,
 	memberchk(file(File), Location),
 	(   memberchk(line(Line), Location)
-	->  Goal = in_pce_thread(emacs(File:Line))
-	;   Goal = in_pce_thread(emacs(File))
+	->  (   memberchk(linepos(LinePos), Location)
+	    ->	Pos = (File:Line:LinePos)
+	    ;	Pos = (File:Line)
+	    )
+	;   Pos = File
 	),
-	Goal.
+	in_pce_thread(emacs(Pos)).
 do_edit_source(Location) :-		% External editor
 	external_edit_command(Location, Command),
 	print_message(informational, edit(waiting_for_editor)),
@@ -307,21 +348,26 @@ editor(_) :-				% No luck
 %		| %d | Line number of the target	  |
 
 
-edit_command(vi,	  '%e +%d ''%f''').
-edit_command(vi,	  '%e ''%f''').
-edit_command(emacs,	  '%e +%d ''%f''').
-edit_command(emacs,	  '%e ''%f''').
+edit_command(vi,	  '%e +%d \'%f\'').
+edit_command(vi,	  '%e \'%f\'').
+edit_command(emacs,	  '%e +%d \'%f\'').
+edit_command(emacs,	  '%e \'%f\'').
 edit_command(notepad,     '"%e" "%f"').
 edit_command(wordpad,     '"%e" "%f"').
 edit_command(uedit32,     '%e "%f/%d/0"').	% ultraedit (www.ultraedit.com)
-edit_command(edit,        '%e %f:%d').		% private stuff
+edit_command(jedit,	  '%e -wait \'%f\' +line:%d').
+edit_command(jedit,	  '%e -wait \'%f\'').
+edit_command(edit,        '%e %f:%d').		% PceEmacs client script
 edit_command(edit,        '%e %f').
 
 edit_command(emacsclient, Command) :- edit_command(emacs, Command).
 edit_command(vim,         Command) :- edit_command(vi,    Command).
 
 substitute(From, ToAtom, Old, New) :-
-	name(ToAtom, To),
+	(   atom(ToAtom)
+	->  atom_codes(ToAtom, To)
+	;   number_codes(ToAtom, To)
+	),
 	append(Pre, S0, Old),
 	append(From, Post, S0) ->
 	append(Pre, To, S1),
@@ -379,7 +425,7 @@ merge_specs(source_file(Path), _, source_file(Path)).
 %%	select_location(+Pairs, +UserSpec, -Location)
 
 do_select_location(Pairs, Spec, Location) :-
-	select_location(Pairs, Spec, Location), !, 		% HOOK
+	select_location(Pairs, Spec, Location), !,		% HOOK
 	Location \== [].
 do_select_location([], Spec, _) :- !,
 	print_message(warning, edit(not_found(Spec))),
@@ -452,12 +498,14 @@ prolog:message(edit(make)) -->
 prolog:message(edit(canceled)) -->
 	[ 'Editor returned failure; skipped make/0 to reload files' ].
 
-edit_specifier(Module:Name/Arity) -->
+edit_specifier(Module:Name/Arity) --> !,
 	[ '~w:~w/~w'-[Module, Name, Arity] ].
-edit_specifier(file(_Path)) -->
+edit_specifier(file(_Path)) --> !,
 	[ '<file>' ].
-edit_specifier(source_file(_Path)) -->
+edit_specifier(source_file(_Path)) --> !,
 	[ '<loaded file>' ].
+edit_specifier(include_file(_Path)) --> !,
+	[ '<included file>' ].
 edit_specifier(Term) -->
 	[ '~p'-[Term] ].
 
