@@ -1,12 +1,10 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2008-2010, University of Amsterdam
-			      Vu University Amsterdam
+    Copyright (C): 2008-2014, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -20,7 +18,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -56,7 +54,11 @@ two additional modules that acts as plugins:
 
     * library(http/http_header)
     Loading this library causes http_open/3 to support the =POST= method
-    in addition to =GET= and =HEAD=.
+    in addition to =GET=, =HEAD= and =DELETE=.
+
+    * library(http/http_ssl_plugin)
+    Loading this library causes http_open/3 HTTPS connections.  Relevant
+    options for SLL certificate handling are handled to ssl_context/3.
 
 Here is a simple example to fetch a web-page:
 
@@ -100,12 +102,29 @@ resource. See also parse_time/2.
 :- meta_predicate
 	http_open(+,-,:).
 
+:- predicate_options(http_open/3, 3,
+		     [ authorization(compound),
+		       final_url(-atom),
+		       header(+atom, -atom),
+		       method(oneof([delete,get,head,post])),
+		       size(-integer),
+		       status_code(-integer),
+		       timeout(number),
+		       post(any),		    % library(http/http_header)
+		       proxy(atom, integer),
+		       proxy_authorization(compound),
+		       request_header(any),
+		       user_agent(atom),
+		       pem_password_hook(callable), % if SSL is loaded
+		       cert_verify_hook(callable)   % if SSL is loaded
+		     ]).
+
 %%	user_agent(-Agent) is det.
 %
 %	Default value for =|User-Agent|=,  can   be  overruled using the
 %	option user_agent(Agent) of http_open/3.
 
-user_agent('SWI-Prolog <http://www.swi-prolog.org>').
+user_agent('SWI-Prolog').
 
 %%	http_open(+URL, -Stream, +Options) is det.
 %
@@ -136,12 +155,15 @@ user_agent('SWI-Prolog <http://www.swi-prolog.org>').
 %	  AtomValue is unified to the empty atom ('').
 %
 %	  * method(+Method)
-%	  One of =get= (default) or =head=.   The  =head= message can be
+%	  One of =get= (default), =head= or =delete=.
+%	  The  =head= message can be
 %	  used in combination with  the   header(Name,  Value) option to
 %	  access information on the resource   without actually fetching
 %	  the resource itself.  The  returned   stream  must  be  closed
-%	  immediately.   If   library(http/http_header)     is   loaded,
-%	  http_open/3 also supports =post=. See the post(Data) option.
+%	  immediately.
+%
+%	  If  library(http/http_header)  is  loaded,   http_open/3  also
+%	  supports =post= and =put=. See the post(Data) option.
 %
 %	  * size(-Size)
 %	  Size is unified with the   integer value of =|Content-Length|=
@@ -164,9 +186,12 @@ user_agent('SWI-Prolog <http://www.swi-prolog.org>').
 %	  Provided if library(http/http_header) is also loaded.  Data is
 %	  handed to http_post_data/3.
 %
-%	  * proxy(+Host, +Port)
+%	  * proxy(+Host:Port)
 %	  Use an HTTP proxy to connect to the outside world.
 %
+%	  * proxy(+Host, +Port)
+%	  Synonym for proxy(+Host:Port).  Deprecated.
+%
 %	  * proxy_authorization(+Authorization)
 %	  Send authorization to the proxy.  Otherwise   the  same as the
 %	  =authorization= option.
@@ -186,14 +211,29 @@ user_agent('SWI-Prolog <http://www.swi-prolog.org>').
 %	@param	URL is either an atom (url) or a list of _parts_.
 %		If this list is provided, it may contain the fields
 %		=scheme=, =user=, =password=, =host=, =port=, =path= and
-%		=search= (where the argument of the latter is a
-%		Name(Value) list).  Only =host= is mandatory.
+%		=search= (where the argument of the latter is a list of
+%		Name(Value) or Name=Value).  Only =host= is mandatory.
+%		The following example below opens the URL
+%		=|http://www.example.com/my/path?q=Hello%20World&lang=en|=.
+%		Note that values must *not* be quoted because the
+%		library inserts the required quites.
+%
+%		==
+%		http_open([ host('www.example.com'),
+%			    path('/my/path'),
+%			    search([ q='Hello world',
+%			             lang=en
+%				   ])
+%			  ])
+%		==
 %
 %	@error existence_error(url, Id)
+%	@see ssl_context/3 for SSL related options if
+%	library(http/http_ssl_plugin) is loaded.
 
 http_open(URL, Stream, QOptions) :-
 	meta_options(is_meta, QOptions, Options),
-	(   atom(URL)
+	(   atomic(URL)
 	->  parse_url_ex(URL, Parts)
 	;   Parts = URL
 	),
@@ -208,21 +248,24 @@ is_meta(pem_password_hook).		% SSL plugin callbacks
 is_meta(cert_verify_hook).
 
 http_open_parts(Parts, Stream, Options0) :-
-	memberchk(proxy(Host, ProxyPort), Options0), !,
+	(   is_list(Options0),
+	    memberchk(proxy(Host, ProxyPort), Options0)
+	;   option(proxy(Host:ProxyPort), Options0)
+	), !,
 	parts_uri(Parts, RequestURI),
 	Options = [visited(Parts)|Options0],
 	open_socket(Host:ProxyPort, In, Out, Options),
 	parts_scheme(Parts, Scheme),
 	default_port(Scheme, DefPort),
-	option(port(Port), Parts, DefPort),
+	url_part(port(Port), Parts, DefPort),
 	host_and_port(Host, DefPort, Port, HostPort),
 	send_rec_header(Out, In, Stream, HostPort, RequestURI, Parts, Options),
 	return_final_url(Options).
 http_open_parts(Parts, Stream, Options0) :-
-	memberchk(host(Host), Parts),
+	option(host(Host), Parts),
 	parts_scheme(Parts, Scheme),
 	default_port(Scheme, DefPort),
-	option(port(Port), Parts, DefPort),
+	url_part(port(Port), Parts, DefPort),
 	parts_request_uri(Parts, RequestURI),
 	Options = [visited(Parts)|Options0],
 	open_socket(Host:Port, SocketIn, SocketOut, Options),
@@ -269,9 +312,9 @@ guarded_send_rec_header(Out, In, Stream, Host, RequestURI, Parts, Options) :-
 	method(Options, MNAME),
 	http_version(Version),
 	format(Out,
-	       '~w ~w HTTP/~w\r\n\
-	       Host: ~w\r\n\
-	       User-Agent: ~w\r\n\
+	       '~w ~w HTTP/~w\r\n\c
+	       Host: ~w\r\n\c
+	       User-Agent: ~w\r\n\c
 	       Connection: close\r\n',
 	       [MNAME, RequestURI, Version, Host, Agent]),
 	x_headers(Options, Out),
@@ -314,10 +357,13 @@ method(Options, MNAME) :-
 	;   domain_error(method, M)
 	).
 
+map_method(delete, 'DELETE').
 map_method(get,  'GET').
 map_method(head, 'HEAD').
 map_method(post, 'POST') :-
 	current_predicate(http_header:http_post_data/3).
+map_method(put, 'PUT') :-
+	current_predicate(http_header:http_post_data/3).
 
 
 %%	x_headers(+Options, +Out) is det.
@@ -343,7 +389,7 @@ x_header(_, _).
 auth_header(basic(User, Password), Header, Out) :- !,
 	format(codes(Codes), '~w:~w', [User, Password]),
 	phrase(base64(Codes), Base64Codes),
-	format(Out, '~w: basic ~s\r\n', [Header, Base64Codes]).
+	format(Out, '~w: Basic ~s\r\n', [Header, Base64Codes]).
 auth_header(Auth, _, _) :-
 	domain_error(authorization, Auth).
 
@@ -438,6 +484,7 @@ redirect_options(Options0, Options) :-
 	;   Options = Options1
 	).
 
+redirect_method(delete).
 redirect_method(get).
 redirect_method(head).
 
@@ -481,22 +528,21 @@ open_socket(Address, In, Out, Options) :-
 	      )),
 	debug(http(open), '\tok ~p --> ~p', [In, Out]),
 	set_stream(In, record_position(false)),
-	(   memberchk(timeout(Timeout), Options)
+	(   option(timeout(Timeout), Options)
 	->  set_stream(In, timeout(Timeout))
 	;   true
 	).
 
 
 return_size(Options, Lines) :-
-	memberchk(size(Size), Options), !,
+	option(size(Size), Options), !,
 	content_length(Lines, Size).
 return_size(_, _).
 
 return_fields([], _).
 return_fields([header(Name, Value)|T], Lines) :- !,
-	atom_codes(Name, Codes),
 	(   member(Line, Lines),
-	    phrase(atom_field(Codes, Value), Line)
+	    phrase(atom_field(Name, Value), Line)
 	->  true
 	;   Value = ''
 	),
@@ -511,9 +557,9 @@ return_fields([_|T], Lines) :-
 %	URL after redirections.
 
 return_final_url(Options) :-
-	memberchk(final_url(URL), Options),
+	option(final_url(URL), Options),
 	var(URL), !,
-	memberchk(visited(Parts), Options),
+	option(visited(Parts), Options),
 	parts_uri(Parts, URL).
 return_final_url(_).
 
@@ -543,7 +589,7 @@ transfer_encoding(Lines, Encoding) :-
 	Encoding = Encoding0.
 
 transfer_encoding(Encoding) -->
-	field("transfer-encoding"),
+	field('transfer-encoding'),
 	rest(Encoding).
 
 %%	read_header(+In:stream, -Code:int, -Comment:atom, -Lines:list) is det.
@@ -560,10 +606,15 @@ read_header(In, Code, Comment, Lines) :-
 	phrase(first_line(Code, Comment), Line),
 	debug(http(open), '~w ~w', [Code, Comment]),
 	read_line_to_codes(In, Line2),
-	rest_header(Line2, In, Lines), !.
+	rest_header(Line2, In, Lines), !,
+	(   debugging(http(open))
+	->  forall(member(HL, Lines),
+		   debug(http(open), '~s', [HL]))
+	;   true
+	).
 read_header(_, 500, 'Invalid reply header', []).
 
-rest_header("", _, []) :- !.		% blank line: end of header
+rest_header([], _, []) :- !.		% blank line: end of header
 rest_header(L0, In, [L0|L]) :-
 	read_line_to_codes(In, L1),
 	rest_header(L1, In, L).
@@ -579,7 +630,7 @@ content_length(Lines, Length) :-
 
 location(Lines, RequestURI) :-
 	member(Line, Lines),
-	phrase(atom_field("location", RequestURI), Line), !.
+	phrase(atom_field(location, RequestURI), Line), !.
 
 first_line(Code, Comment) -->
 	"HTTP/", [_], ".", [_],
@@ -593,17 +644,21 @@ atom_field(Name, Value) -->
 	rest(Value).
 
 content_length(Len) -->
-	field("content-length"),
+	field('content-length'),
 	integer(Len).
 
-field([]) -->
+field(Name) -->
+	{ atom_codes(Name, Codes) },
+	field_codes(Codes).
+
+field_codes([]) -->
 	":",
 	skip_blanks.
-field([H|T]) -->
+field_codes([H|T]) -->
 	[C],
 	{ match_header_char(H, C)
 	},
-	field(T).
+	field_codes(T).
 
 match_header_char(C, C) :- !.
 match_header_char(C, U) :-
@@ -687,7 +742,7 @@ check_authorization(basic(User, Password)) :-
 	must_be(atom, User),
 	must_be(atom, Password).
 
-%%	authorization(+URL, -Authorization) is semdet.
+%%	authorization(+URL, -Authorization) is semidet.
 %
 %	True if Authorization must be supplied for URL.
 %
@@ -710,8 +765,8 @@ authorization(URL, Authorization) :-
 add_authorization(_, Options, Options) :-
 	option(authorization(_), Options), !.
 add_authorization(Parts, Options0, Options) :-
-	option(user(User), Parts),
-	option(password(Passwd), Parts),
+	url_part(user(User), Parts),
+	url_part(password(Passwd), Parts),
 	Options = [authorization(basic(User,Passwd))|Options0].
 add_authorization(Parts, Options0, Options) :-
 	stored_authorization(_, _) ->	% quick test to avoid work
@@ -729,7 +784,7 @@ add_authorization(_, Options, Options).
 parse_url_ex(URL, [uri(URL)|Parts]) :-
 	uri_components(URL, Components),
 	phrase(components(Components), Parts),
-	(   memberchk(host(_), Parts)
+	(   option(host(_), Parts)
 	->  true
 	;   domain_error(url, URL)
 	).
@@ -788,39 +843,39 @@ uri_request_uri(Components) -->
 %%	parts_authority(+Parts, -Authority) is semidet.
 
 parts_scheme(Parts, Scheme) :-
-	option(scheme(Scheme), Parts), !.
+	url_part(scheme(Scheme), Parts), !.
 parts_scheme(Parts, Scheme) :-		% compatibility with library(url)
-	option(protocol(Scheme), Parts), !.
+	url_part(protocol(Scheme), Parts), !.
 parts_scheme(_, http).
 
 parts_authority(Parts, Auth) :-
-	option(authority(Auth), Parts), !.
+	url_part(authority(Auth), Parts), !.
 parts_authority(Parts, Auth) :-
-	option(host(Host), Parts, _),
-	option(port(Port), Parts, _),
-	option(user(User), Parts, _),
-	option(password(Password), Parts, _),
+	url_part(host(Host), Parts, _),
+	url_part(port(Port), Parts, _),
+	url_part(user(User), Parts, _),
+	url_part(password(Password), Parts, _),
 	uri_authority_components(Auth,
 				 uri_authority(User, Password, Host, Port)).
 
 parts_request_uri(Parts, RequestURI) :-
-	memberchk(request_uri(RequestURI), Parts), !.
+	option(request_uri(RequestURI), Parts), !.
 parts_request_uri(Parts, RequestURI) :-
-	option(path(Path), Parts, /),
+	url_part(path(Path), Parts, /),
 	ignore(parts_search(Parts, Search)),
 	uri_data(path, Data, Path),
 	uri_data(search, Data, Search),
 	uri_components(RequestURI, Data).
 
 parts_search(Parts, Search) :-
-	memberchk(query_string(Search), Parts), !.
+	option(query_string(Search), Parts), !.
 parts_search(Parts, Search) :-
-	memberchk(search(Fields), Parts), !,
+	option(search(Fields), Parts), !,
 	uri_query_components(Search, Fields).
 
 
 parts_uri(Parts, URI) :-
-	memberchk(uri(URI), Parts), !.
+	option(uri(URI), Parts), !.
 parts_uri(Parts, URI) :-
 	parts_scheme(Parts, Scheme),
 	ignore(parts_authority(Parts, Auth)),
@@ -830,6 +885,20 @@ parts_uri(Parts, URI) :-
 	uri_data(authority, Data, Auth),
 	uri_components(URI, Data).
 
+url_part(Part, Parts) :-
+	Part =.. [Name,Value],
+	Gen =.. [Name,RawValue],
+	option(Gen, Parts), !,
+	Value = RawValue.
+
+url_part(Part, Parts, Default) :-
+	Part =.. [Name,Value],
+	Gen =.. [Name,RawValue],
+	(   option(Gen, Parts)
+	->  Value = RawValue
+	;   Value = Default
+	).
+
 
 		 /*******************************
 		 *	      COOKIES		*
@@ -843,7 +912,7 @@ update_cookies(_, _, _) :-
 	predicate_property(http:update_cookies(_,_,_), number_of_clauses(0)), !.
 update_cookies(Lines, Parts, Options) :-
 	(   member(Line, Lines),
-	    phrase(atom_field("set_cookie", CookieData), Line),
+	    phrase(atom_field('set_cookie', CookieData), Line),
 	    http:update_cookies(CookieData, Parts, Options),
 	    fail
 	;   true
@@ -865,7 +934,7 @@ update_cookies(Lines, Parts, Options) :-
 %		http:open_options/2.
 %
 %	    http:open_options(Parts, Options) :-
-%		memberchk(host(Host), Parts),
+%		option(host(Host), Parts),
 %		Host \== localhost,
 %		Options = [proxy('proxy.local', 3128)].
 %	    ==
@@ -876,10 +945,18 @@ update_cookies(Lines, Parts, Options) :-
 %	open stream to the HTTP server, Parts is the broken-down request
 %	(see uri_components/2) and Options is the list of options passed
 %	to http_open.  The predicate is called as if using ignore/1.
+%
+%	@see complements http:update_cookies/3.
+%	@see library(http/http_cookies) implements cookie handling on
+%	top of these hooks.
 
-%%	update_cookies(+CookieData, +Parts, +Options) is semidet.
+%%	http:update_cookies(+CookieData, +Parts, +Options) is semidet.
 %
 %	Update the cookie database.  CookieData  is   the  value  of the
 %	=|Set-Cookie|= field, Parts is  the   broken-down  request  (see
 %	uri_components/2) and Options is the list   of options passed to
 %	http_open.
+%
+%	@see complements http:write_cookies
+%	@see library(http/http_cookies) implements cookie handling on
+%	top of these hooks.
